from datetime import datetime
import requests
from bs4 import BeautifulSoup as beautifulsoup


def getfloat(number):
    number = number.replace(" ", "")
    return 0.0 if number == "" else float(number.split("*")[0])


def getsat(satellite):
    return satellite.replace("-", "").lower()


def getmonth(month):
    month = month.split(' ')
    month = '%s %s' % (month[0][:3], month[1])
    return datetime.strptime(month, "%b %Y")


class CountsShift(object):
    sats = {'goes13': 32.0}

    def coefficient(self, sat):
        return CountsShift.sats[sat]


class SpaceMeasurement(object):
    sats = {'goes13': 29.0}

    def coefficient(self, sat):
        return SpaceMeasurement.sats[sat]


class PreLaunch(object):

    def __init__(self):
        table = self.table()
        self.coefficients = self.resume_coefficients(table)

    def table(self):
        page = requests.get("http://www.star.nesdis.noaa.gov/smcd/spb/fwu/"
                            "homepage/GOES_Imager_Vis_PreCal.php")
        pq = beautifulsoup(page.text)
        return (pq.select("table")[1]).select("tr")

    def resume_coefficients(self, table):
        co = {}
        for r in table[1:]:
            childs = r.findChildren()
            co[getsat(childs[0].text)] = [getfloat(e.text)
                                          for e in childs[2:]]
        return co

    def coefficient(self, sat):
        return self.coefficients[getsat(sat)]


class PostLaunch(object):

    def __init__(self):
        table = self.table()
        self.sats = self.satellites(table)
        self.months = self.resume_coefficients(table)

    def table(self):
        page = requests.get('http://www.star.nesdis.noaa.gov/smcd/spb/fwu/'
                            'homepage/GOES_Imager_Vis_OpCal.php')
        pq = beautifulsoup(page.text)
        return (pq.select("table")[2]).select("tr")

    def satellites(self, table):
        sats = table[1:2]
        sats = [getsat(s.text) for s in sats[0].select('td')
                if not s.text == ""]
        return sats

    def resume_coefficients(self, table):
        co = {}
        for r in table[2:]:
            childs = r.findChildren()
            co[getmonth(childs[0].text)] = [getfloat(e.text)
                                            for e in childs[1:]]
        return co

    def get_month(self, dt):
        month = dt.month
        days = [datetime(dt.year, dt.month, d)
                    for d in range(1, dt.day + 1)]
        tuesdays = lambda d: d.weekday() == 1
        days = filter(tuesdays, days)
        date = datetime(dt.year, month, 1)
        if len(days) < 2 or date not in self.months:
            month -= 1
        date = datetime(dt.year, month, 1)
        return date

    def coefficient(self, dt, sat):
        s = self.sats.index(sat)
        dt = self.get_month(dt)
        # TODO: Select the appropiate month according to the day of the month.
        return self.months[dt][s]


def calibration(date, satellite):
    """
    Return the calibration dictionary.

    Keyword arguments:
    satellite -- the name of the satellite.
    date --  the datetime of an image.
    """
    counts_shift = CountsShift()
    space_measurement = SpaceMeasurement()
    prelaunch = PreLaunch()
    postlaunch = PostLaunch()
    return {
        'counts_shift': counts_shift.coefficient(satellite),
        'space_measurement': space_measurement.coefficient(satellite),
        'prelaunch': prelaunch.coefficient(satellite),
        'postlaunch': postlaunch.coefficient(date, satellite)
    }
